/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

/*
 *
 * vm_types.h --
 *
 *    basic data types.-- part of driver-only distribution.
 */


#ifndef _VM_TYPES_H_
#define _VM_TYPES_H_

#define INCLUDE_ALLOW_USERLEVEL
#define INCLUDE_ALLOW_MONITOR
#define INCLUDE_ALLOW_MODULE
#define INCLUDE_ALLOW_VMKERNEL
#define INCLUDE_ALLOW_VMK_MODULE
#define INCLUDE_ALLOW_DISTRIBUTE
#include "includeCheck.h"

#include "vm_basic_types.h"
#include "vm_basic_defs.h"
#include "vm_assert.h"

typedef unsigned MemHandle;
typedef unsigned PhysMemMappingHandle;

/*
 *----------------------------------------------------------------------
 *
 * IsGoodBA --
 *
 *      Basic checks for validity of a BA
 *
 * Right now this function only checks that the low 20 bits of a BA
 * are all zeroes, which should be enough to detect people who
 * incorrectly tried to use BA as an arithmetic type.
 *
 * Results:
 *      TRUE iif BA seems valid.
 *----------------------------------------------------------------------
 */

static INLINE Bool
IsGoodBA(BA ba)
{
   return (ba & 0xfffff) == 0;
}

static INLINE unsigned
BA_PAGE_OFFSET(BA ba)
{
   ASSERT(IsGoodBA(ba));
   return (uint32)ba >> (32 - PAGE_SHIFT);
}

/*
 *----------------------------------------------------------------------
 *
 * MemOffset_2_BA --
 *
 *      Construct a BA from a MemHandle and a byte offset
 *
 * Results:
 *      constructed BA
 *----------------------------------------------------------------------
 */

static INLINE BA
MemOffset_2_BA(MemHandle slot, uint64 offset)
{
   return ((uint64)slot << 56) | (offset << 20);
}

/*
 *----------------------------------------------------------------------
 *
 * BA_2_MemHandle --
 *
 *      Extract the MemHandle part of a BA
 *
 * Results:
 *      MemHandle
 *----------------------------------------------------------------------
 */

static INLINE MemHandle
BA_2_MemHandle(BA ba)
{
   ASSERT(IsGoodBA(ba));
   return (MemHandle)(ba >> 56);
}

/*
 *----------------------------------------------------------------------
 *
 * BA_2_Offset --
 *
 *      Extract the byte offset part of a BA
 *
 * Results:
 *      byte offset
 *----------------------------------------------------------------------
 */

static INLINE uint64
BA_2_Offset(BA ba)
{
   ASSERT(IsGoodBA(ba));
   return (ba >> 20) & ((((uint64)1)<<36)-1);	/* 36-bit byte offset */
}

/*
 *----------------------------------------------------------------------
 *
 * MemPage_2_BPN --
 *
 *      Construct a BPN from a MemHandle and a PageNum
 *
 * Results:
 *      constructed BPN
 *----------------------------------------------------------------------
 */

static INLINE BPN
MemPage_2_BPN(MemHandle slot, PageNum page)
{
   return (slot << 24) | page;
}

/*
 *----------------------------------------------------------------------
 *
 * BPN_2_MemHandle --
 *
 *      Extract the MemHandle part of a BPN
 *
 * Results:
 *      MemHandle
 *----------------------------------------------------------------------
 */

static INLINE MemHandle
BPN_2_MemHandle(BPN bpn)
{
   return bpn >> 24;
}

/*
 *----------------------------------------------------------------------
 *
 * BPN_2_PageNum --
 *
 *      Extract the PageNum part of a BPN
 *
 * Results:
 *      PageNum
 *----------------------------------------------------------------------
 */

static INLINE uint32
BPN_2_PageNum(BPN bpn)
{
   return bpn & 0xffffff;		/* 24-bit page offset */
}

/*
 *----------------------------------------------------------------------
 *
 * BA_2_BPN --
 *
 *      Extract the BPN part of a BA
 *
 * Results:
 *      BPN
 *----------------------------------------------------------------------
 */

static INLINE BPN
BA_2_BPN(BA ba)
{
   ASSERT(IsGoodBA(ba));
   return (BPN)(ba >> 32);
}

/*
 *----------------------------------------------------------------------
 *
 * BPN_Offset_2_BA --
 *
 *      Construct a BA from a BPN and a byte offset within that page
 *
 * Results:
 *      constructed BA
 *----------------------------------------------------------------------
 */

static INLINE BA
BPN_Offset_2_BA(BPN bpn, uint32 offset)
{
   ASSERT(offset < PAGE_SIZE);
   return ((uint64)bpn << 32) | (offset << 20);
}


typedef void  *HostThread;         /* Host's thread identifier. */

#define ACCESS_MODE_READ      (1 << 0)
#define ACCESS_MODE_WRITE     (1 << 1)
#define ACCESS_MODE_READWRITE (ACCESS_MODE_READ | ACCESS_MODE_WRITE)
#define ACCESS_MODE_NONE      (0)
#define ACCESS_MODE_MASK      (ACCESS_MODE_READ | ACCESS_MODE_WRITE)

typedef unsigned AccessMode;
typedef uint8    AccessMode8;

/*
 * IRQ
 */

#define NOIRQ 0xFF
/* 
 * 8 'real' PCI IRQs (PIRQA-PIRQD for 440BX + PIRQE-PIRQH for modern chipsets)
 * + 8 'special' (SMI, SCI, NMI, etc) 
 */
#define MAX_VIRQS 32
#define LEGAL_IRQ(_irq) ( ((_irq) == NOIRQ) || ((_irq) < MAX_VIRQS) )

typedef enum {
   PIRQA = 16,
   PIRQB,
   PIRQC,
   PIRQD,

   PIRQE,
   PIRQF,
   PIRQG,
   PIRQH,

   IRQ_SMI,
   IRQ_SCI,
   IRQ_NMI,

   IRQ_LAST
} PIRQ;

typedef unsigned TraceHandle;


#endif // ifndef _VM_TYPES_H_
